# ID3Tag Wrapper Class - Copyright (c) 2003. Alastair Tse. <acnt2@cam.ac.uk>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Changes:
#  
# Version 0.1 - Initial Release (09 Oct 2003)
#
# Todo:
# 
# - Work out id3_tag_render and what it actually wants
# - Catch errors and throw proper exceptions
# - Need to be careful of left over refs to tags/fields/frames when file is closed.
#

from libid3tag import *
from types import *

class ID3File:
    def __init__(self, file, mode=ID3_FILE_MODE_READONLY):
	if type(file) == StringType:
	    self.file = id3_file_open(file, mode)
	else:
	    self.file = id3_file_fdopen(file, mode)

    def close(self):
	if self.file:
	    return id3_file_close(self.file)

    def update(self):
	assert self.file
	return id3_file_update(self.file)
    
    def tag(self):
	assert self.file
	return ID3Tag(id3_file_tag(self.file))
    
    def __del__(self):
	self.close()

class ID3Tag:
    def __init__(self, tag=None):
	if tag:
	    self.tag = tag
	else:
	    self.tag = id3_tag_new()

    def __del__(self):
	if self.tag:
	    id3_tag_delete(self.tag)
	    
    def version(self):
	assert self.tag
	return id3_tag_version(self.tag)
    
    def options(self, opt, value):
	assert self.tag
	return id3_tag_options(self.tag, opt, value)
    
    def setlength(self, length):
	assert self.tag
	id3_tag_setlength(self.tag, length)
	
    def clearframes(self):
	assert self.tag
	id3_tag_clearframes(self.tag)
	
    def attachframe(self, frame):
	assert self.tag
	return id3_tag_attachframe(self.tag, frame)
	
    def detachframe(self, frame):
	assert self.tag
	return id3_tag_detachframe(self.tag, frame)
    
    def findframe(self, name, number):
	assert self.tag
	return ID3Frame(id3_tag_findframe(self.tag, name, number), "obj")
    
    def query(self, bytes):
	return id3_tag_query(bytes)
    
    def parse(self, bytes, length):
	return ID3Tag(id3_tag_parse(bytes))
    
    # TESTME: does it only return the byte string?
    def render(self):
	pass
    
class ID3Frame:
    def __init__(self, name, mode="name"):
	if mode == "name":
	    self.frame = id3_frame_new(name)
	else:
	    self.frame = name
    
    def __del__(self):
	if self.frame:
	    id3_frame_delete(self.frame)
	    
    def field(self, num):
	assert self.frame
	return ID3Field(id3_frame_field(self.frame, num))
    
class ID3Field:
    def __init__(self, obj):
	self.field = obj
	
    def field_type(self):
	assert self.field
	return id3_field_type(self.field)
    
    def setint(self, num):
	assert self.field
	return id3_field_setint(self.field, num)
    
    def settextencoding(self, enc):
	assert self.field
	return id3_field_settextencoding(self.field, enc)
    
    def setstrings(self, strings):
	assert self.field
	return id3_field_setstrings(self.field, strings)
    
    def addstring(self, s):
	assert self.field
	return id3_field_addstring(self.field, s)
    
    def setlanguage(self, lang):
	assert self.field
	return id3_field_setlanguage(self.field, lang)
    
    def setlatin1(self, s):
	assert self.field
	return id3_field_setlatin1(self.field, s)
    
    def setfulllatin1(self, s):
	assert self.field
	return id3_field_setfulllatin1(self.field, s)
    
    def setstring(self, s):
	assert self.field
	return id3_field_setstring(self.field, s)
    
    def setfullstring(self, s):
	assert self.field
	return id3_field_setfullstring(self.field, s)    
    
    def setframeid(self, s):
	assert self.field
	return id3_field_setframeid(self.field, s)
    
    def setbinarydata(self, bin):
	assert self.field
	return id3_field_setbinarydata(self.field, bin)
    
    def getint(self):
	assert self.field
	return id3_field_getint(self.field)
    
    def getlatin1(self):
	assert self.field
	return id3_field_getlatin1(self.field)
    
    def getfulllatin1(self):
	assert self.field
	return id3_field_getfulllatin1(self.field)
    
    def getstring(self):
	assert self.field
	return id3_field_getstring(self.field)
    
    def getfullstring(self):
	assert self.field
	return id3_field_getfullstring(self.field)
    
    def getnstrings(self):
	assert self.field
	return id3_field_getnstrings(self.field)
    
    def getstrings(self, num):
	assert self.field
	return id3_field_getstrings(self.field, num)
    
    def getframeid(self):
	assert self.field
	return id3_field_getframeid(self.field)
    
    def getbinarydata(self):
	assert self.field
	return id3_field_getbinarydata(self.field)    
    
class ID3Genre:
    
    def index(self, num):
	return id3_genre_index(num)
    
    def name(self, s):
	return id3_genre_name(s)
    
    def number(self, name):
	return id3_genre_number(name)
